==> flutter_snow_base-0.1.5%2B1.tar.gz: lib/src/utils/network_bound_resources.dart <==
import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'resource.dart';

class NetworkBoundResources<ResultType, RequestType> {
  StreamController<Resource<ResultType>> _result;

  Future<Resource<ResultType>> asFuture({
    FutureOr<RequestType> Function() loadFromDb,
    bool shouldFetch(RequestType data),
    @required FutureOr<RequestType> Function() createCall,
    ResultType processResponse(RequestType result),
    FutureOr saveCallResult(RequestType item),
  }) {
    assert(
      RequestType == ResultType ||
          (!(RequestType == ResultType) && processResponse != null),
      "You need to specify the `processResponse` when the types are different",
    );
    processResponse ??= (value) => value as ResultType;
    return Resource.asFuture<ResultType>(() async {
      final value = loadFromDb == null ? null : await loadFromDb();
      final _shouldFetch = shouldFetch == null ? true : shouldFetch(value);
      return processResponse(
        _shouldFetch
            ? await _fetchFromNetwork(createCall, saveCallResult, value)
            : value,
      );
    });
  }

  Stream<Resource<ResultType>> asStream({
    Stream<RequestType> Function() loadFromDbStream,
    Future<RequestType> Function() loadFromDbFuture,
    @required bool shouldFetch(RequestType data),
    @required FutureOr<RequestType> Function() createCall,
    ResultType processResponse(RequestType result),
    @required FutureOr saveCallResult(RequestType item),
  }) {
    assert(
      RequestType == ResultType ||
          (!(RequestType == ResultType) && processResponse != null),
      "You need to specify the `processResponse` when the types are different",
    );
    assert(
      loadFromDbStream == null && loadFromDbFuture != null ||
          loadFromDbStream != null && loadFromDbFuture == null,
      "You need to specify at least and only one of `loadFromDbFuture` or `loadFromDbStream` to load your data",
    );
    processResponse ??= (value) => value as ResultType;

    StreamSubscription localListener;

    _result = StreamController<Resource<ResultType>>.broadcast(
      onCancel: () {
        if (!_result.hasListener) {
          _result?.close();
          localListener?.cancel();
        }
      },
    );

    _result.sink.add(Resource.loading());

    void _fetchData(
        FutureOr<RequestType> event, Sink<Resource<ResultType>> sink) async {
      event = await event;
      final eventResult = processResponse(event);
      if (shouldFetch(event)) {
        // print("Fetch data and call loading");
        sink.add(Resource.loading(data: eventResult));
        try {
          var result = await _fetchFromNetwork(createCall, saveCallResult);
          // print("Fetching success");
          sink.add(Resource.success(data: processResponse(result)));
        } on Exception catch (e) {
          // print("Fetching failed");
          sink.add(Resource.failed(data: null, error: e));
        }
      } else {
        // print("Fetching data its not necessary");
        sink.add(Resource.success(data: eventResult));
      }
    }

    if (loadFromDbStream != null) {
      final localStream = loadFromDbStream().transform(
        StreamTransformer<RequestType, Resource<ResultType>>.fromHandlers(
          handleData: _fetchData,
        ),
      );

      localListener = localStream.listen(_result.sink.add);
    } else {
      _fetchData(loadFromDbFuture(), _result.sink);
    }

    // print("Call loading...");

    return _result.stream;
  }

  Future<RequestType> _fetchFromNetwork(
      Future<RequestType> Function() createCall,
      Future Function(RequestType item) saveCallResult,
      [RequestType unconfirmedResult]) async {
    if (saveCallResult != null && unconfirmedResult != null) {
      saveCallResult(unconfirmedResult);
    }
    return await createCall().then((value) async {
      if (value != unconfirmedResult && saveCallResult != null) {
        await saveCallResult(value);
      }
      return value;
    });
  }
}

==> flutter_snow_base-0.1.6%2B1.tar.gz: lib/src/utils/network_bound_resources.dart <==
import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'resource.dart';

class NetworkBoundResources<ResultType, RequestType> {
  StreamController<Resource<ResultType>> _result;

  Future<Resource<ResultType>> asFuture({
    FutureOr<RequestType> Function() loadFromDb,
    bool shouldFetch(RequestType data),
    @required FutureOr<RequestType> Function() createCall,
    ResultType processResponse(RequestType result),
    FutureOr saveCallResult(RequestType item),
  }) {
    assert(
      RequestType == ResultType ||
          (!(RequestType == ResultType) && processResponse != null),
      "You need to specify the `processResponse` when the types are different",
    );
    processResponse ??= (value) => value as ResultType;
    return Resource.asFuture<ResultType>(() async {
      final value = loadFromDb == null ? null : await loadFromDb();
      final _shouldFetch = shouldFetch == null ? true : shouldFetch(value);
      return processResponse(
        _shouldFetch
            ? await _fetchFromNetwork(createCall, saveCallResult, value)
            : value,
      );
    });
  }

  Stream<Resource<ResultType>> asStream({
    Stream<RequestType> Function() loadFromDbStream,
    Future<RequestType> Function() loadFromDbFuture,
    @required bool shouldFetch(RequestType data),
    @required FutureOr<RequestType> Function() createCall,
    ResultType processResponse(RequestType result),
    @required FutureOr saveCallResult(RequestType item),
  }) {
    assert(
      RequestType == ResultType ||
          (!(RequestType == ResultType) && processResponse != null),
      "You need to specify the `processResponse` when the types are different",
    );
    assert(
      loadFromDbStream == null && loadFromDbFuture != null ||
          loadFromDbStream != null && loadFromDbFuture == null,
      "You need to specify at least and only one of `loadFromDbFuture` or `loadFromDbStream` to load your data",
    );
    processResponse ??= (value) => value as ResultType;

    StreamSubscription localListener;

    _result = StreamController<Resource<ResultType>>.broadcast(
      onCancel: () {
        if (!_result.hasListener) {
          _result?.close();
          localListener?.cancel();
        }
      },
    );

    _result.sink.add(Resource.loading());

    void _fetchData(
        FutureOr<RequestType> event, Sink<Resource<ResultType>> sink) async {
      event = await event;
      final eventResult = processResponse(event);
      if (shouldFetch(event)) {
        // print("Fetch data and call loading");
        sink.add(Resource.loading(data: eventResult));
        try {
          var result = await _fetchFromNetwork(createCall, saveCallResult);
          // print("Fetching success");
          sink.add(Resource.success(data: processResponse(result)));
        } on Exception catch (e) {
          // print("Fetching failed");
          sink.add(Resource.failed(data: null, error: e));
        }
      } else {
        // print("Fetching data its not necessary");
        sink.add(Resource.success(data: eventResult));
      }
    }

    if (loadFromDbStream != null) {
      final localStream = loadFromDbStream().transform(
        StreamTransformer<RequestType, Resource<ResultType>>.fromHandlers(
          handleData: _fetchData,
        ),
      );

      localListener = localStream.listen(_result.sink.add);
    } else {
      _fetchData(loadFromDbFuture(), _result.sink);
    }

    // print("Call loading...");

    return _result.stream;
  }

  Future<RequestType> _fetchFromNetwork(
      Future<RequestType> Function() createCall,
      Future Function(RequestType item) saveCallResult,
      [RequestType unconfirmedResult]) async {
    if (saveCallResult != null && unconfirmedResult != null)
      saveCallResult(unconfirmedResult);
    return await createCall().then((value) async {
      if (value != unconfirmedResult && saveCallResult != null) {
        await saveCallResult(value);
      }
      return value;
    });
  }
}
